// Jacek Fedorynski <jfedor@jfedor.org>
// More info here: http://blog.jfedor.org/2013/10/morse-code-usb-keyboard.html

#include <CapacitiveSensor.h>

const long UNIT_TIME = 75; // dit duration in milliseconds

const int CAPSENSE_SENDPIN = 10;
const int CAPSENSE_RECVPIN = 11;
const int LED_PIN = 13;
const int BUZZER_PIN = A3;

const int ALPHABET_SIZE = 47;

char* letters[] = {
  ".-", "A",
  "-...", "B",
  "-.-.", "C",
  "-..", "D",
  ".", "E",
  "..-.", "F",
  "--.", "G",
  "....", "H",
  "..", "I",
  ".---", "J",
  "-.-", "K",
  ".-..", "L",
  "--", "M",
  "-.", "N",
  "---", "O",
  ".--.", "P",
  "--.-", "Q",
  ".-.", "R",
  "...", "S",
  "-", "T",
  "..-", "U",
  "...-", "V",
  ".--", "W",
  "-..-", "X",
  "-.--", "Y",
  "--..", "Z",
  ".----", "1",
  "..---", "2",
  "...--", "3",
  "....-", "4",
  ".....", "5",
  "-....", "6",
  "--...", "7",
  "---..", "8",
  "----.", "9",
  "-----", "0",
  ".-.-.-", ".",
  "--..--", ",",
  "---...", ":",
  "..--..", "?",
  ".----.", "'",
  "-....-", "-",
  "-..-.", "/",
  "-.--.", "(",
  "-.--.-", ")",
  ".-..-.", "\"",
  "-...-", "="
};

CapacitiveSensor capsense = CapacitiveSensor(CAPSENSE_SENDPIN, CAPSENSE_RECVPIN);

int prevstate = 0;
unsigned long prevtime;

void setup()                    
{
   pinMode(LED_PIN, OUTPUT);
   pinMode(BUZZER_PIN, OUTPUT);
   digitalWrite(BUZZER_PIN, HIGH); // my buzzer makes a sound when the signal pin is driven low
   prevtime = millis();
   Keyboard.begin();
}

const int DITSLEN = 10;

char dits[DITSLEN];
int i = 0;

void emitLetter() {
  dits[i] = '\0';
  Serial.println(dits);
  for (int j=0; j<ALPHABET_SIZE; j++) {
    if (strcmp(dits, letters[2*j]) == 0) {
      Serial.println(letters[2*j+1]);
      Keyboard.print(letters[2*j+1]);
      break;
    }
  }
  i = 0;
}

void loop()                    
{
    long senseval =  capsense.capacitiveSensor(30);
    int state = senseval > 1000;
    digitalWrite(LED_PIN, state ? HIGH : LOW);
    digitalWrite(BUZZER_PIN, state ? LOW : HIGH);
    
    unsigned long t = millis();
    unsigned long len = t-prevtime;

    if (prevstate != state) {
      if (prevstate) {
        if (len < 10) {
          // ignore
        } else if (len < UNIT_TIME*2) {
          // dit
          if (i < DITSLEN-1) {
            dits[i++] = '.';
          }
        } else {
          // dah
          if (i < DITSLEN - 1) {
            dits[i++] = '-';
          }
        }
      } else {
        if (len < UNIT_TIME*2) {
          // intra-letter gap
        } else if (len < UNIT_TIME*5.33) {
          // gap between letters
          emitLetter();
        }
      }
      
      prevstate = state;
      prevtime = t;
    } else {
      if (!prevstate && (i > 0) && (len >= UNIT_TIME*5.33)) {
        // gap between words
        emitLetter();
        Keyboard.print(" ");
      }
    }
}
